#!/usr/bin/env python
# _*_coding:utf-8_*_
"""
@Time   :  2021/8/10 22:13
@Author :  Qinghua Wang
@Email  :  597935261@qq.com
"""

import math
from collections import namedtuple
from typing import List

import zbar

try:
    import cv2
except ModuleNotFoundError:
    print("Warning,func show_info can not be used when cv2 is not available")

Position = namedtuple("Position", ["left_top", "left_bottom", "right_bottom", "right_top"])


class BarcodeRes:
    """
    BarcodeRes

    text : text of utf-8
    type : barcode type
    location : barcode point list
    rect : bounding box of location
    ori_orientation : zbar inner orientation (Only used for get points)
    orientation : orientation degree
    position : namedtuple with fields ["left_top", "left_bottom", "right_bottom", "right_top"]
    """

    def __init__(self, x: zbar.Symbol):
        self.text = x.data
        self.type = str(x.type)
        self.location = x.location
        self.rect = get_bbox(x.location)
        self.ori_orientation = str(x.orientation)
        if self.ori_orientation == "LEFT":  # for LEFT
            self.position = Position._make([self.location[0], self.location[3], self.location[2], self.location[1]])
        else:
            self.position = Position._make(self.location)

        self.orientation = get_clockwise_orientation(self.position.left_bottom, self.position.left_top, "degree")

    def __repr__(self):
        return str(self.__dict__)


def get_clockwise_orientation(start_p, end_p, return_format="degree"):
    """
    calc clockwise orientation
    :param start_p: start point
    :param end_p: end point
    :param return_format: degree or radian
    :return:
    """
    d_x = end_p[0] - end_p[0]
    d_y = end_p[1] - start_p[1]
    if d_y == 0:
        if d_x >= 0:
            res = math.pi / 2
        else:
            res = -math.pi / 2
    else:
        res = math.atan(d_x / d_y)
    if return_format == "degree":
        res = res / math.pi * 180
    return round(res)


def get_bbox(p_list):
    """

    :param p_list:
    :return:
    """
    x_list, y_list = [item[0] for item in p_list], [item[1] for item in p_list]
    p_left_top = (min(x_list), min(y_list))
    p_right_bottom = (max(x_list), max(y_list))
    x_center = (p_left_top[0] + p_right_bottom[0]) / 2
    y_center = (p_left_top[1] + p_right_bottom[1]) / 2
    width = p_right_bottom[0] - p_left_top[0]
    height = p_right_bottom[1] - p_left_top[1]
    return x_center, y_center, width, height


def decode(img):
    """
    get BarCode decode result
    :param img: cv2 image (np array)(gray is better)
    :return:
    """
    scanner = zbar.ImageScanner()
    scanner.parse_config('enable')
    height, width = img.shape[:2]
    raw = img.tobytes()
    image = zbar.Image(width, height, 'Y800', raw)
    scanner.scan(image)
    res = [BarcodeRes(x) for x in image]
    return res


def show_info(barcode_list: List[BarcodeRes], image):
    """

    :param barcode_list:
    :param image:
    :return:
    """
    for barcode in barcode_list:
        (x, y, w, h) = barcode.rect
        cv2.rectangle(image, (int(x - w / 2), int(y - h / 2)), (int(x + w / 2), int(y + h / 2)),
                      (255, 255, 0), 10)
        cv2.putText(image, barcode.text, (int(x - w / 2), int(y - h / 2)),
                    cv2.FONT_HERSHEY_SIMPLEX, 0.5, (0, 0, 255), 1)
    cv2.imshow("res", image)
    cv2.waitKey()


if __name__ == '__main__':
    image_path = "../test.png"
    img = cv2.imread(image_path)
    print(decode(cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)))
    show_info(decode(cv2.imread(image_path, cv2.IMREAD_GRAYSCALE)), img)



#!/usr/bin/env python
# _*_coding:utf-8_*_
"""
@Time   :  2021/2/7 18:51
@Author :  Qinghua Wang
@Email  :  597935261@qq.com
"""
import ctypes
import ctypes.util
import os
import sys

CUR_DIR = os.path.realpath(os.path.dirname(__file__))
OS_PLATFORM = sys.platform

print("os platform is {}".format(OS_PLATFORM))
HAVE_X = False

try:
    import setuptools
    from setuptools import setup, Extension
    from setuptools.command.install import install
    from setuptools.command.build_ext import build_ext


    class MyBuildExt(build_ext):
        """
        MyBuildExt
        """

        def build_extensions(self):
            """

            :return:
            """
            self.compiler.dll_libraries = ["gdi32", "winmm"] if OS_PLATFORM == "win32" and not HAVE_X else []
            build_ext.build_extensions(self)


    setuptools_opts = dict(cmdclass={"install": install, "build_ext": MyBuildExt})

except ImportError as err:
    print(err)
    from distutils.core import setup
    from distutils.extension import Extension

    setuptools_opts = {}


def use_config_template():
    """
    use_config_template
    :return:
    """
    config_path = os.path.join(CUR_DIR, "src", "config.h")
    if OS_PLATFORM == "win32":
        target_os = "windows"
    else:
        target_os = "linux"
    template_file = os.path.join(CUR_DIR, "config_template", target_os, "config.h")
    if not os.path.exists(config_path):
        print("use template config.h in {}".format(template_file))
        with open(config_path, "w", encoding="utf-8") as f:
            with open(template_file, "r") as f1:
                c = f1.read()
            f.write(c)


use_config_template()


def get_c_file(target_dir):
    """
    to find the .c file
    :param target_dir:
    :return:
    """
    return [os.path.join(target_dir, x) for x in os.listdir(target_dir) if x.endswith(".c")]


ZBAR_SOURCE_FOR_PYTHON = "zbar_python"
SRCS = get_c_file("src/zbar") + \
       get_c_file("src/zbar/decoder") + \
       get_c_file("src/zbar/video") + \
       get_c_file("src/zbar/qrcode") + get_c_file(ZBAR_SOURCE_FOR_PYTHON) + [
           os.path.join("src/zbar/processor", "lock.c")]

INCLUDE = ['./src', './src/zbar', "./src/zbar/decoder", "./src/zbar/qrcode", "./src/zbar/processor",
           "./src/zbar/window", ZBAR_SOURCE_FOR_PYTHON]

print(SRCS)


def adjust_c_source_file(SRCS):
    """
    adjust source file
    """
    # pdf417 can not use now
    SRCS.remove(os.path.join("src/zbar/decoder", "pdf417.c"))
    if OS_PLATFORM == "win32":
        SRCS.append(os.path.join("src/zbar/processor", "win.c"))
        if not HAVE_X:
            SRCS.append(os.path.join("src/zbar/window", "win.c"))
            SRCS.append(os.path.join("src/zbar/window", "dib.c"))
    else:
        SRCS.append(os.path.join("src/zbar/processor", "posix.c"))
        if not HAVE_X:
            SRCS.append(os.path.join("src/zbar/processor", "null.c"))
            SRCS.append(os.path.join("src/zbar/window", "null.c"))


def has_libc_iconv():
    """
    find iconv
    :return:
    """
    if os.name != 'posix':
        return False
    libc = ctypes.CDLL(ctypes.util.find_library('c'))
    return hasattr(libc, 'iconv')


adjust_c_source_file(SRCS)
print(SRCS)

# don't try to link to standalone iconv library if it's already in libc
# (iconv is in glibc, but on OS X one needs a stanalone libiconv)
LIBS = [] if has_libc_iconv() else ['iconv']
zbar = Extension('zbar', sources=SRCS, include_dirs=INCLUDE, libraries=LIBS)

setup(name='zbar-lite',
      version='0.23.92',
      description='zbar lite package only support scan image and recognize barcode',
      long_description=open(os.path.join(CUR_DIR, "README.md"), encoding="utf-8").read(),
      long_description_content_type="text/markdown",
      url='https://github.com/wqh17101/zbar_lite',
      author='Qinghua Wang',
      author_email='597935261@qq.com',
      ext_modules=[zbar],
      packages=['zbar_helper'],
      license='MIT',
      **setuptools_opts)
